package org.one.stone.soup.server;

import java.io.FileInputStream;
import java.net.ServerSocket;
import java.security.KeyStore;
import java.security.Security;

import javax.net.ServerSocketFactory;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;

public abstract class SecureServer extends Server
{
	// from http://www.onjava.com/pub/a/onjava/2001/05/03/java_security.html?page=3
	
	 private String storeName;
	 private String storePassword;
	 private String keyPassword;
	 private boolean authenticateClient;
	
	 public void initialiseKeyStore(String storeName,String storePassword,String keyPassword,boolean authenticateClient)
	 {
		 this.storeName = storeName;
		 this.storePassword = storePassword;
		 this.keyPassword = keyPassword;
		 this.authenticateClient = authenticateClient;
	 }
	 
	public ServerSocket getServerSocket(int port,int bufferSize,String address) throws Exception
	{
		if(storeName==null)
		{
			throw new Exception("You must call initialiseKeyStore before creating a sercure server.");
		}
		
		  // Make sure that JSSE is available
		  Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
		  // A keystore is where keys and certificates are kept
		  // Both the keystore and individual private keys should be password protected
		  KeyStore keystore = KeyStore.getInstance("JKS");
		  keystore.load(new FileInputStream(storeName), storePassword.toCharArray());
		  // A KeyManagerFactory is used to create key managers
		  KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
		  // Initialize the KeyManagerFactory to work with our keystore
		  kmf.init(keystore, keyPassword.toCharArray());
		  // An SSLContext is an environment for implementing JSSE
		  // It is used to create a ServerSocketFactory
		  SSLContext sslc = SSLContext.getInstance("SSLv3");
		  // Initialize the SSLContext to work with our key managers
		  sslc.init(kmf.getKeyManagers(), null, null);
		  // Create a ServerSocketFactory from the SSLContext
		  ServerSocketFactory ssf = sslc.getServerSocketFactory();
		  // Socket to me
		  SSLServerSocket serverSocket =
		   (SSLServerSocket) ssf.createServerSocket(port);
		  // Authenticate the client?
		  serverSocket.setNeedClientAuth(authenticateClient);
		  // Return a ServerSocket on the desired port (443)
		  return serverSocket;
	}
}
