package org.one.stone.soup.server.smtp.dummy;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.SocketException;

import org.one.stone.soup.entity.KeyValuePair;
import org.one.stone.soup.file.FileHelper;
import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.xml.XmlElement;

public class SmtpSession {

	private SmtpServer server;
	private Socket socket;
	
	private BufferedReader in;
	private PrintWriter out;
	
	private boolean dataMode = false;
	private boolean inHeader = false;
	private XmlElement email;
	private StringBuffer data;
	
	public SmtpSession(SmtpServer server,Socket socket) throws Exception
	{
		this.server = server;
		this.socket = socket;
		in = new BufferedReader( new InputStreamReader(socket.getInputStream()) );
		out = new PrintWriter( socket.getOutputStream() );
	}
	
	public void start()
	{
		send("220 Dummy Simple Mail Transfer Service Ready");
		
		try{
			String line = in.readLine();
			while(line != null)
			{
				System.out.println(line);
				if(dataMode)
				{
					readEMail(line);
				}
				else
				{
					process( StringArrayHelper.parseSpacedFields(line) );
				}
				
				//send("250 Dummy");
				line = in.readLine();
			}
		}
		catch(SocketException e)
		{
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		stop();
	}
	
	public void send(String text)
	{
		System.out.println("> "+text);
		out.print(text+"\r\n");
		out.flush();
	}
	
	public void process(String[] args)
	{
		if(args.length==0)
		{
			return;
		}
		
		if(args[0].equals("EHLO"))
		{
			send("250 Dummy");
		}
		else if(args[0].equals("MAIL"))
		{
			send("250 OK");
		}
		else if(args[0].equals("RCPT"))
		{
			send("250 OK");
		}
		else if(args[0].equals("DATA"))
		{
			send("354 Start mail input; end with <CRLF>.<CRLF>");
			dataMode=true;
			inHeader=true;
			email = new XmlElement("email");
		}
		else if(args[0].equals("QUIT"))
		{
			send("221 Dummy Service closing transmission channel");
		}
	}
	
	public void readEMail(String line) throws Exception
	{
		if(line.equals("."))
		{
			email.addChild("contents").addValue(data.toString());
			send("250 OK");
			dataMode=false;
			saveEmail();
			return;
		}
		if(inHeader && line.length()==0)
		{
			inHeader=false;
			data = new StringBuffer();
			return;
		}
		
		if(inHeader)
		{
			KeyValuePair headerAttribute = KeyValuePair.parseKeyValuePair(line, ": ");
			email.addChild(headerAttribute.key).addValue(headerAttribute.value);
			
			if(headerAttribute.key.equals("Content-Type") && headerAttribute.value.equals("multipart/alternative;"))
			{
				email.getElementByName("Content-Type").setValue("multipart/alternative;\n"+in.readLine());
			}
		}
		else
		{
			data.append(line);
		}
	}
	
	public void saveEmail()
	{
		String from = email.getElementByName("From").getValue().trim();
		if(from.indexOf('<')!=-1)
		{
			from = from.substring( from.indexOf('<')+1,from.indexOf('>') );
		}
		String to = email.getElementByName("To").getValue().trim();
		if(to.indexOf('<')!=-1)
		{
			to = to.substring( to.indexOf('<')+1,to.indexOf('>') );
		}
		String fileName =  "/From-"+from+" To-"+to+" Subject-"+email.getElementByName("Subject").getValue().trim();
        
		StringBuffer newFileName = new StringBuffer();
        for(int loop=0;loop<fileName.length();loop++)
        {
            if(Character.isLetterOrDigit( fileName.charAt(loop) ))
            {
                newFileName.append( fileName.charAt(loop) );
            }
            else
            {
                newFileName.append('-');
            }
        }
        fileName = newFileName.toString();
        
		fileName = fileName+".email";
		fileName = server.getRoot()+fileName; 
			
		int index = 2;
		File file = new File(fileName+".xml");
		while(file.exists())
		{
			file = new File(fileName+index+".xml");
			index++;
		}
		
		FileHelper.buildFile( file.getAbsolutePath(),email.toXml() );
		
		System.out.println("EMail saved as: "+file.getAbsolutePath());
	}
	
	public void stop()
	{
		try{
		socket.close();
		}
		catch(Exception e)
		{}
		server.remove(this);
	}
}
