package test.org.one.stone.soup.file;

import java.io.File;

import junit.framework.TestCase;

import org.one.stone.soup.file.FileChangeListener;
import org.one.stone.soup.file.FilesWatch;

public class FilesWatchTest extends TestCase {

	
	public void testTriggeredWhenFilesChange()
	{
		try{
			File file1 = File.createTempFile("file1-watch","test");
			File file2 = File.createTempFile("file2-watch","test");
			
			class FileWatcher implements FileChangeListener
			{
				public boolean changed = false;
				
				public boolean waitForChange(long time)
				{
					long count = time/10;
					while(count>0)
					{
						count--;
						if(changed)
						{
							return true;
						}
						try{ Thread.sleep(10);}catch(Exception e){}
					}
					return false;
				}
				
				public void fileChanged(File changedFile)
				{
					changed = true;
				}
			}
			FileWatcher listener = new FileWatcher();
			
			FilesWatch filesWatch = new FilesWatch("Files Watch Test",500);
			filesWatch.addFile(file1);
			filesWatch.addFile(file2);
			filesWatch.addListener( listener );
			
			file1.setLastModified( file1.lastModified()+1000 );
			boolean result = listener.waitForChange(1000);
			assertEquals(true,result);
			
			listener.changed = false;
			file2.setLastModified( file2.lastModified()+1000 );
			result = listener.waitForChange(1000);
			assertEquals(true,result);
		}
		catch(Exception e)
		{
			fail("Exception thrown: "+e);
		}
	}
	
	public void testTriggeredWhenFolderChanges()
	{
		try{
			File file = File.createTempFile("file-watch","test");
			File folder = file.getParentFile();
			
			class FileWatcher implements FileChangeListener
			{
				public boolean changed = false;
				
				public boolean waitForChange(long time)
				{
					long count = time/10;
					while(count>0)
					{
						count--;
						if(changed)
						{
							return true;
						}
						try{ Thread.sleep(10);}catch(Exception e){}
					}
					return false;
				}
				
				public void fileChanged(File changedFile)
				{
					changed = true;
				}
			}
			FileWatcher listener = new FileWatcher();
			
			FilesWatch filesWatch = new FilesWatch("Files Watch Test",500);
			filesWatch.addFolder(folder);
			filesWatch.addListener( listener );
			
			file.setLastModified( file.lastModified()+1000 );
			
			boolean result = listener.waitForChange(1000);
			
			assertEquals(true,result);
		}
		catch(Exception e)
		{
			fail("Exception thrown: "+e);
		}
	}	
}
